@@ -86,6 +86,7 @@ use vars qw($VERSION %DecoderFor);
 
 ### System modules:
 use IPC::Open2;
+use IO::Select;
 use FileHandle;
 
 ### Kit modules:
@@ -125,7 +126,7 @@ use Carp;
 );
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = "5.427";
+$VERSION = "5.500";
 
 ### Me:
 my $ME = 'MIME::Decoder';
@@ -335,10 +336,6 @@ are some other methods you'll need to know about:
 
 =over 4
 
-=cut
-
-#------------------------------
-
 =item decode_it INSTREAM,OUTSTREAM
 
 I<Abstract instance method.>
@@ -366,8 +363,6 @@ sub decode_it {
     die "attempted to use abstract 'decode_it' method!";
 }
 
-#------------------------------
-
 =item encode_it INSTREAM,OUTSTREAM
 
 I<Abstract instance method.>
@@ -395,8 +390,6 @@ sub encode_it {
     die "attempted to use abstract 'encode_it' method!";
 }
 
-#------------------------------
-
 =item filter IN, OUT, COMMAND...
 
 I<Class method, utility.>
@@ -417,25 +410,65 @@ so you can specify COMMAND as a single argument or as an array.
 
 =cut
 
-sub filter {
-    my ($self, $in, $out, @cmd) = @_;
-    my $buf = '';
+sub filter
+{
+	my ($self, $in, $out, @cmd) = @_;
+	my $buf = '';
 
-    ### Open pipe:
-    STDOUT->flush;       ### very important, or else we get duplicate output!
-    my $kidpid = open2(\*CHILDOUT, \*CHILDIN, @cmd) || die "open2 failed: $!";
+	### Open pipe:
+	STDOUT->flush;  ### very important, or else we get duplicate output!
 
-    ### Write all:
-    while ($in->read($buf, 2048)) { print CHILDIN $buf }
-    close \*CHILDIN;
+	my $kidpid = open2(my $child_out, my $child_in, @cmd) || die "@cmd: open2 failed: $!";
 
-    ### Read all:
-    while (read(\*CHILDOUT, $buf, 2048)) { $out->print($buf) }
-    close \*CHILDOUT;
+	### We have to use select() for doing both reading and writing.
+	my $rsel = IO::Select->new( $child_out );
+	my $wsel = IO::Select->new( $child_in  );
 
-    ### Wait for it:
-    waitpid($kidpid,0) or die "couldn't reap child $kidpid";
-    1;
+	while (1) {
+
+		### Wait for one hour; if that fails, it's too bad.
+		my ($read, $write) = IO::Select->select( $rsel, $wsel, undef, 3600);
+
+		if( !defined $read && !defined $write ) {
+			kill 1, $kidpid;
+			waitpid $kidpid, 0;
+			die "@cmd: select failed: $!";
+		}
+
+		### If can read from child:
+		if( my $fh = shift @$read ) {
+			if( $fh->sysread(my $buf, 1024) ) {
+				$out->print($buf);
+			} else {
+				$rsel->remove($fh);
+				$fh->close();
+			}
+		}
+
+		### If can write to child:
+		if( my $fh = shift @$write ) {
+			if($in->read(my $buf, 1024)) {
+				local $SIG{PIPE} = sub {
+					warn "got SIGPIPE from @cmd";
+					$wsel->remove($fh);
+					$fh->close();
+				};
+				$fh->syswrite( $buf );
+			} else {
+				$wsel->remove($fh);
+				$fh->close();
+			}
+		}
+
+		### If both $child_out and $child_in are done:
+		last unless ($rsel->count() || $wsel->count());
+	}
+
+	### Wait for it:
+	waitpid($kidpid, 0) == $kidpid or die "@cmd: couldn't reap child $kidpid";
+	### Check if it failed:
+	$? == 0 or die "@cmd: bad exit status: \$? = $?";
+	1;
 }
 
 
