@@ -234,7 +234,6 @@ use MIME::Tools qw(:config :msgs :utils);
 use MIME::Head;
 use MIME::Body;
 use MIME::Decoder;
-use IO::Lines;
 
 @ISA = qw(Mail::Internet);
 
@@ -246,7 +245,7 @@ use IO::Lines;
 #------------------------------
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = "5.426";
+$VERSION = "5.500";
 
 ### Boundary counter:
 my $BCount = 0;
@@ -646,7 +645,16 @@ sub build {
     ### Add other MIME fields:
     $head->replace('Content-transfer-encoding', $encoding) if $encoding;
     $head->replace('Content-description', $desc)           if $desc;
-    $head->replace('Content-id', $id)                      if defined($id);
+
+    # Content-Id value should be surrounded by < >, but versions before 5.428
+    # did not do this.  So, we check, and add if the caller has not done so
+    # already.
+    if( defined $id ) {
+	if( $id !~ /^<.*>$/ ) {
+		$id = "<$id>";
+	}
+	$head->replace('Content-id', $id);
+    }
     $head->replace('MIME-Version', '1.0')                  if $top;
 
     ### Add the X-Mailer field, if top level (use default value if not given):
@@ -686,7 +694,7 @@ external data in bodyhandles is I<not> copied to new files!
 Changing the data in one entity's data file, or purging that entity,
 I<will> affect its duplicate.  Entities with in-core data probably need
 not worry.
-'
+
 =cut
 
 sub dup {
@@ -1163,15 +1171,14 @@ sub make_singlepart {
     return 'ALREADY' if !$self->is_multipart;      ### already a singlepart?
     return '0' if ($self->parts > 1);              ### can this even be done?
 
-    ### Do it:
+    # Get rid of all our existing content info
+    my $tag;
+    foreach $tag (grep {/^content-/i} $self->head->tags) {
+        $self->head->delete($tag);
+    }
+
     if ($self->parts == 1) {    ### one part
 	my $part = $self->parts(0);
-	my $tag;
-
-	### Get rid of all our existing content info:
-	foreach $tag (grep {/^content-/i} $self->head->tags) {
-	    $self->head->delete($tag);
-	}
 
 	### Populate ourselves with any content info from the part:
 	foreach $tag (grep {/^content-/i} $part->head->tags) {
@@ -1279,8 +1286,17 @@ sub remove_sig {
     my $self = shift;
     my $nlines = shift;
 
-    ### Handle multiparts:
-    $self->is_multipart and return $self->{ME_Parts}[0]->remove_sig(@_);
+    # If multipart, we only attempt to remove the sig from the first
+    # part.  This is usually a good assumption for multipart/mixed, but
+    # may not always be correct.  It is also possibly incorrect on
+    # multipart/alternative (both may have sigs).
+    if( $self->is_multipart ) {
+	my $first_part = $self->parts(0);
+	if( $first_part ) {
+            return $first_part->remove_sig(@_);
+	}
+	return undef;
+    }
 
     ### Refuse non-textual unless forced:
     textual_type($self->head->mime_type)
@@ -1799,8 +1815,15 @@ sub print_body {
 	my $boundary = $self->head->multipart_boundary;
 
 	### Preamble:
-	my $preamble = join('', @{ $self->preamble || $DefPreamble });
-	$out->print("$preamble\n") if ($preamble ne '' or $self->preamble);
+	my $plines = $self->preamble;
+	if (defined $plines) {
+	    # Defined, so output the preamble if it exists (avoiding additional
+	    # newline as per ticket 60931)
+	    $out->print( join('', @$plines) . "\n") if (@$plines > 0);
+	} else {
+	    # Undefined, so use default preamble
+	    $out->print( join('', @$DefPreamble) . "\n" );
+	}
 
 	### Parts:
 	my $part;
@@ -2220,6 +2243,9 @@ how we work.
 
 =back
 
+=head1 SEE ALSO
+
+L<MIME::Tools>, L<MIME::Head>, L<MIME::Body>, L<MIME::Decoder>, L<Mail::Internet>
 
 =head1 AUTHOR
 
