@@ -106,8 +106,8 @@ Ready?  Ok...
 =head2 Miscellaneous examples
 
     ### Convert a Mail::Internet object to a MIME::Entity:
-    @lines = (@{$mail->header}, "\n", @{$mail->body});
-    $entity = $parser->parse_data(\@lines);
+    my $data = join('', (@{$mail->header}, "\n", @{$mail->body}));
+    $entity = $parser->parse_data(\$data);
 
 
 
@@ -129,10 +129,8 @@ require 5.004;
 use strict;
 use vars (qw($VERSION $CAT $CRLF));
 
-### Built-in modules:
-use IO::ScalarArray  1.114;
+### core Perl modules
 use IO::File;
-use IO::InnerFile;
 use File::Spec;
 use File::Path;
 use Config qw(%Config);
@@ -148,31 +146,6 @@ use MIME::Parser::Reader;
 use MIME::Parser::Filer;
 use MIME::Parser::Results;
 
-
-#============================================================
-#
-# A special kind of inner file that we can virtually print to.
-#
-package MIME::Parser::InnerFile;
-
-use vars qw(@ISA);
-@ISA = qw(IO::InnerFile);
-
-sub print {
-    shift->add_length(length(join('', @_)));
-    1;
-}
-
-sub PRINT  {
-    shift->{LG} += length(join('', @_));
-    1;
-}
-
-#============================================================
-
-package MIME::Parser;
-
-
 #------------------------------
 #
 # Globals
@@ -180,7 +153,7 @@ package MIME::Parser;
 #------------------------------
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = "5.426";
+$VERSION = "5.500";
 
 ### How to catenate:
 $CAT = '/bin/cat';
@@ -245,7 +218,6 @@ sub init {
     $self->{MP5_ParseNested}     = 'NEST';
     $self->{MP5_TmpToCore}       = 0;
     $self->{MP5_IgnoreErrors}    = 1;
-    $self->{MP5_UseInnerFiles}   = 0;
     $self->{MP5_UUDecode}        = 0;
     $self->{MP5_MaxParts}        = -1;
     $self->{MP5_TmpDir}          = undef;
@@ -577,7 +549,19 @@ sub process_preamble {
 
     ### Parse preamble:
     my @saved;
-    $rdr->read_lines($in, \@saved);
+    my $data = '';
+    open(my $fh, '>', \$data) or die $!;
+    $rdr->read_chunk($in, $fh, 1);
+    close $fh;
+
+    # Ugh.  Horrible.  If the preamble consists only of CRLF, squash it down
+    # to the empty string.  Else, remove the trailing CRLF.
+    if( $data =~ m/^[\r\n]\z/ ) {
+	@saved = ('');
+    } else {
+	$data =~ s/[\r\n]\z//;
+        @saved = split(/^/, $data);
+    }
     $ent->preamble(\@saved);
     1;
 }
@@ -638,8 +622,11 @@ sub process_header {
     my $hdr_rdr = $rdr->spawn;
     $hdr_rdr->add_terminator("");
     $hdr_rdr->add_terminator("\r");           ### sigh
-    $hdr_rdr->read_lines($in, \@headlines);
-    foreach (@headlines) { s/[\r\n]+\Z/\n/ }  ### fold
+
+    my $headstr = '';
+    open(my $outfh, '>:scalar', \$headstr) or die $!;
+    $hdr_rdr->read_chunk($in, $outfh);
+    close $outfh;
 
     ### How did we do?
     if ($hdr_rdr->eos_type eq 'DELIM') {
@@ -649,27 +636,16 @@ sub process_header {
     ($hdr_rdr->eos_type eq 'DONE') or
 	$self->error("unexpected end of header\n");
 
-    ### Cleanup bogus header lines.
-    ###    Some folks like to parse mailboxes, so the header will start
-    ###    with "From " or ">From ".  Tolerate this by removing both kinds
-    ###    of lines silently (can't we use Mail::Header for this, and try
-    ###    and keep the envelope?).  Ditto for POP.
-    while (@headlines) {
-	if    ($headlines[0] =~ /^>?From /) {    ### mailbox
-	    $self->whine("skipping bogus mailbox 'From ' line");
-	    shift @headlines;
-	}
-	elsif ($headlines[0] =~ /^\+OK/) {       ### POP3 status line
-	    $self->whine("skipping bogus POP3 '+OK' line");
-	    shift @headlines;
-	}
-	else { last }
+    ### Extract the header (note that zero-size headers are admissible!):
+    open(my $readfh, '<:scalar', \$headstr) or die $!;
+    $head->read( $readfh );
+
+    unless( $readfh->eof() ) {
+	# Not entirely correct, since ->read consumes the line it gives up on.
+	# it's actually the line /before/ the one we get with ->getline
+	$self->error("couldn't parse head; error near:\n", $readfh->getline());
     }
 
-    ### Extract the header (note that zero-size headers are admissible!):
-    $head->extract(\@headlines);
-    @headlines and
-	$self->error("couldn't parse head; error near:\n",@headlines);
 
     ### If desired, auto-decode the header as per RFC 2047
     ###    This shouldn't affect non-encoded headers; however, it will decode
@@ -789,16 +765,8 @@ sub process_singlepart {
     }
     else {
 
-	### Can we read real fast?
-	if ($self->{MP5_UseInnerFiles} &&
-	    $in->can('seek') && $in->can('tell')) {
-	    $self->debug("using inner file");
-	    $ENCODED = MIME::Parser::InnerFile->new($in, $in->tell, 0);
-	}
-	else {
-	    $self->debug("using temp file");
-	    $ENCODED = $self->new_tmpfile();
-	}
+	$self->debug("using temp file");
+	$ENCODED = $self->new_tmpfile();
 
 	### Read encoded body until boundary (or EOF)...
 	$self->process_to_bound($in, $rdr, $ENCODED);
@@ -1051,7 +1019,7 @@ sub process_part {
     if (not defined $head) {
        $self->debug("bogus empty part");
        $head = $self->interface('HEAD_CLASS')->new;
-       $head->mime_type('text/plain; charset=US-ASCII');
+       $head->mime_type('text/plain');
        $ent->head($head);
        $ent->bodyhandle($self->new_body_for($head));
        $ent->bodyhandle->open("w")->close or die "$ME: can't close: $!";
@@ -1103,23 +1071,34 @@ sub process_part {
 =item parse_data DATA
 
 I<Instance method.>
-Parse a MIME message that's already in core.
+Parse a MIME message that's already in core.  This internally creates an "in
+memory" filehandle on a Perl scalar value using PerlIO
+
 You may supply the DATA in any of a number of ways...
 
 =over 4
 
 =item *
 
-B<A scalar> which holds the message.
+B<A scalar> which holds the message.  A reference to this scalar will be used
+internally.
 
 =item *
 
-B<A ref to a scalar> which holds the message.  This is an efficiency hack.
+B<A ref to a scalar> which holds the message.  This reference will be used
+internally.
 
 =item *
 
-B<A ref to an array of scalars.>  They are treated as a stream
-which (conceptually) consists of simply concatenating the scalars.
+B<DEPRECATED>
+
+B<A ref to an array of scalars.>  The array is internally concatenated into a
+temporary string, and a reference to the new string is used internally.
+
+It is much more efficient to pass in a scalar reference, so please consider
+refactoring your code to use that interface instead.  If you absolutely MUST
+pass an array, you may be better off using IO::ScalarArray in the calling code
+to generate a filehandle, and passing that filehandle to I<parse()>
 
 =back
 
@@ -1138,10 +1117,11 @@ sub parse_data {
     } elsif( ref $data eq 'SCALAR' ) {
         $io = IO::File->new($data, '<:');
     } elsif( ref $data eq 'ARRAY' ) {
-	# Unfortunately, if they give us an array, we have to keep
-	# using it.  We don't really want to make a copy.
-	# TODO: I think we're stuck keeping this one for now.
-        $io = IO::ScalarArray->new($data);
+	# Passing arrays is deprecated now that we've nuked IO::ScalarArray
+	# but for backwards compatibility we still support it by joining the
+	# array lines to a scalar and doing scalar IO on it.
+	my $tmp_data = join('', @$data);
+	$io = IO::File->new(\$tmp_data, '<:');
     } else {
         croak "parse_data: wrong argument ref type: ", ref($data);
     }
@@ -1521,28 +1501,32 @@ sub tmp_to_core {
 
 =item use_inner_files [YESNO]
 
+I<REMOVED>.
+
 I<Instance method.>
-If you are parsing from a handle which supports seek() and tell(),
-then we can avoid tmpfiles completely by using IO::InnerFile, if so
-desired: basically, we simulate a temporary file via pointers
-to virtual start- and end-positions in the input stream.
 
-If YESNO is false (the default), then we will not use IO::InnerFile.
-If YESNO is true, we use IO::InnerFile if we can.
-With no argument, just returns the current setting.
+MIME::Parser no longer supports IO::InnerFile, but this method is retained for
+backwards compatibility.  It does nothing.
 
-B<Note:> inner files are slower than I<real> tmpfiles,
-but possibly faster than I<in-core> tmpfiles... so your choice for
-this option will probably depend on your choice for
-L<tmp_to_core()|/tmp_to_core> and the kind of input streams you are
-parsing.
+The original reasoning for IO::InnerFile was that inner files were faster than
+"in-core" temp files.  At the time, the "in-core" tempfile support was
+implemented with IO::Scalar from the IO-Stringy distribution, which used the
+tie() interface to wrap a scalar with the appropriate IO::Handle operations.
+The penalty for this was fairly hefty, and IO::InnerFile actually was faster.
+
+Nowadays, MIME::Parser uses Perl's built in ability to open a filehandle on an
+in-memory scalar variable via PerlIO.  Benchmarking shows that IO::InnerFile is
+slightly slower than using in-memory temporary files, and is slightly faster
+than on-disk temporary files.  Both measurements are within a few percent of
+each other.  Since there's no real benefit, and since the IO::InnerFile abuse
+was fairly hairy and evil ("writes" to it were faked by extending the size of
+the inner file with the assumption that the only data you'd ever ->print() to
+it would be the line from the "outer" file, for example) it's been removed.
 
 =cut
 
 sub use_inner_files {
-    my ($self, $yesno) = @_;
-    $self->{MP5_UseInnerFiles} = $yesno if (@_ > 1);
-    $self->{MP5_UseInnerFiles};
+	return 0;
 }
 
 =back
@@ -1803,21 +1787,6 @@ Optimum settings:
 				    general you want it set to 1)
     output_to_core()           0   (will be MUCH faster)
     tmp_to_core()              0   (will be MUCH faster)
-    use_inner_files()          0   (if tmp_to_core() is 0;
-				    use 1 otherwise)
-
-B<File I/O is much faster than in-core I/O.>
-Although it I<seems> like slurping a message into core and
-processing it in-core should be faster... it isn't.
-Reason: Perl's filehandle-based I/O translates directly into
-native operating-system calls, whereas the in-core I/O is
-implemented in Perl.
-
-B<Inner files are slower than real tmpfiles, but faster than in-core ones.>
-If speed is your concern, that's why
-you should set use_inner_files(true) if you set tmp_to_core(true):
-so that we can bypass the slow in-core tmpfiles if the input stream
-permits.
 
 B<Native I/O is much faster than object-oriented I/O.>
 It's much faster to use E<lt>$fooE<gt> than $foo-E<gt>getline.
@@ -1851,10 +1820,6 @@ Optimum settings:
     output_to_core()           0   (will use MUCH less memory)
 				    tmp_to_core is 1)
     tmp_to_core()              0   (will use MUCH less memory)
-    use_inner_files()          *** (no real difference, but set it to 1
-				    if you *must* have tmp_to_core set to 1,
-				    so that you avoid in-core tmpfiles)
-
 
 =head2 Maximizing tolerance of bad MIME
 
@@ -1872,7 +1837,6 @@ Optimum settings:
 				    but often you want it set to 1 anyway).
     output_to_core()           *** (doesn't matter)
     tmp_to_core()              *** (doesn't matter)
-    use_inner_files()          *** (doesn't matter)
 
 
 =head2 Avoiding disk-based temporary files
@@ -1890,17 +1854,9 @@ Optimum settings:
     extract_nested_messages()  *** (doesn't matter)
     output_to_core()           *** (doesn't matter)
     tmp_to_core()              1
-    use_inner_files()          1
-
-B<If we can use them, inner files avoid most tmpfiles.>
-If you parse from a seekable-and-tellable filehandle, then the internal
-process_to_bound() doesn't need to extract each part into a temporary
-buffer; it can use IO::InnerFile (B<warning:> this will slow down
-the parsing of messages with large attachments).
 
 B<You can veto tmpfiles entirely.>
-If you might not be parsing from a seekable-and-tellable filehandle,
-you can set L<tmp_to_core()|/tmp_to_core> true: this will always
+You can set L<tmp_to_core()|/tmp_to_core> true: this will always
 use in-core I/O for the buffering (B<warning:> this will slow down
 the parsing of messages with large attachments).
 
@@ -2029,7 +1985,9 @@ my attention.>
 
 =back
 
+=head1 SEE ALSO
 
+L<MIME::Tools>, L<MIME::Head>, L<MIME::Body>, L<MIME::Entity>, L<MIME::Decoder>
 
 =head1 AUTHOR
 
